from re import A


class DataStruct():
    class LinkedList():
        class Node():
            def __init__(self,value=None,next=None,prev=None):
                self.value = value
                self.next = next
                self.prev = prev
           
        def __init__(self):
            self.__head = None
            self.__tail = None
            self.__len = 0
        
        def isempty(self):
            return self.__head == None
        
        def length(self):
            return self.__len
        
        def append(self,value):
            if self.__head == None:
                self.__head = self.Node(value)
                self.__tail = self.__head
            else:
                self.__tail.next = self.Node(value,prev=self.__tail)
                self.__tail = self.__tail.next
            self.__len += 1
            
        def insert(self,idx,value):
            if idx == 0:
                self.add(value)
                return
            elif idx == self.length()-1:
                self.append(value)
                return
            pointer = self.__head
            for i in range(idx-1):
                pointer = pointer.next
            serach = self.Node(value,pointer.next,pointer)
            serach.next.prev = serach
            pointer.next = serach
            self.__len += 1
            
        def add(self,value):
            if self.__head == None:
                node = self.Node(value)
                self.__head = node
                self.__tail = node
            else:
                node = self.Node(value,self.__head)
                self.__head.prev = node
                self.__head = node
            self.__len += 1
            
        def remove(self,idx):
            pointer = self.__head
            for i in range(idx):
                pointer = pointer.next
            if pointer == self.__head:
                self.__head = self.__head.next
                if pointer.next:
                    pointer.next.prev = None
                return
            if pointer.next == None:
                pointer.prev.next = pointer.next
                return
            pointer.prev.next = pointer.next
            pointer.next.prev = pointer.prev
            self.__len -= 1
            
        def delete(self,value):
            pointer = self.__head
            for i in range(self.length()-1):
                if pointer.value == value:
                    self.remove(i)
                    return
                pointer = pointer.next
            
        def serach(self,value):
            pointer = self.__head
            for i in range(self.length()-1):
                if pointer.value == value:
                    return i
                else:
                    pointer = pointer.next
            return False
        
        def isexist(self,value):
            pointer = self.__head
            for i in range(self.length()-1):
                if pointer.value == value:
                    return True
                else:
                    pointer = pointer.next
            return False
        
        def setvalue(self,idx,value):
            pointer = self.__head
            for i in range(idx):
                pointer = pointer.next
            pointer.value = value
            return False
        
        
        def show(self):
            import prettytable
            tb = prettytable.PrettyTable()
            tb.field_names = ["Value","Next","Prev"]
            pointer = self.__head
            while pointer:
                if pointer.next == None:
                    if pointer.prev == None:
                        tb.add_row([pointer.value,None,None])
                    else:
                        tb.add_row([pointer.value,None,pointer.prev.value])
                else:
                    if pointer.prev == None:
                        tb.add_row([pointer.value,pointer.next.value,None])
                    else:
                        tb.add_row([pointer.value,pointer.next.value,pointer.prev.value])
                pointer = pointer.next
            return tb
        
    class Queue():
        def __init__(self,maxlen):
            self.__queuelst = []
            self.__front = 0
            self.__rear = -1
            self.__maxlen = maxlen
            self.__len = 0
            
        def isempty(self):
            return self.__len == 0
        
        def isfull(self):
            return self.__maxlen == self.__len
        
        def length(self):
            return self.__len
        
        def put(self,value):
            if self.isfull == True:
                return False
            self.__queuelst.append(value)
            self.__rear += 1
            self.__len += 1
            
        def leave(self):
            if self.isempty():
                return False
            del self.__queuelst[self.__front]
            self.__len -= 1
            
        def show(self):
            import prettytable
            tb = prettytable.PrettyTable()
            tb.field_names = ['QueueIdx'+str(i) for i in range(0,self.__len)]
            tb.add_row(self.__queuelst)
            return tb
        
    class Stack():
        def __init__(self,maxhei):
            self.__top = 0
            self.__stacklst = []
            self.__maxhei = maxhei
            
        def isempty(self):
            return self.__top == 0
        
        def height(self):
            return self.__top
        
        def isfull(self):
            return self.__top == self.__maxhei
        
        def push(self,value):
            if self.isfull():
                return False
            self.__stacklst.append(value)
            self.__top += 1
            
        def pop(self):
            if self.isempty():
                return False
            self.__stacklst.pop()
            
        def show(self):
            import prettytable
            tb = prettytable.PrettyTable()
            self.__stacklst.reverse()
            tb.add_column("Top",self.__stacklst)
            return tb
            
        


class Digit():
    def __init__(self):
        pass
    
    def plus(self,num1,num2):
        return num1+num2
    
    def minus(self,num1,num2):
        return num1-num2
    
    def mult(self,num1,num2):
        return num1*num2
    
    def div(self,num1,num2,reverse=False):
        return num1/num2 if not reverse else num2/num1
    
    def mod(self,num1,num2):
        return num1%num2
    
    def abs(self,num):
        return num if num>=0 else -num
    
    def opp(self,num):
        return -num
    
    def gcd(self,num1,num2):
        gcdlst = []
        for i in range(1,self.min(num1,num2)+1):
            if num1 % i == 0 and num2 % i == 0:
                gcdlst.append(i)
        return self.maxlst(gcdlst)
    
    def lcm(self,num1,num2):
        for i in range(self.max(num1,num2),num1*num2+1):
            if i % num1 ==0 and i % num2 ==0:
                return i
            
    def avg(self,lst):
        sum = 0
        for i in lst:
            sum += i
        return sum/len(lst)
    
    def max(self,num1,num2):
        return num1 if num1>num2 else num2
    
    def min(self,num1,num2):
        return num1 if num1<num2 else num2
        
    def sum(self,lst):
        sumn = 0
        for i in lst:
            sumn += i
        return sumn
    
    def square(self,num):
        return self.pow(num,2)
    
    def cube(self,num):
        return self.pow(num,3)
    
    def e(self,num):
        length = 0
        num = str(num)
        if num[0] == '-':
            new = '-{}.'.format(num[1])
        elif num[0].isdigit():
            new = '{}.'.format(num[0])
        for i in range(1 if num[0].isdigit else 2,len(num)):
            new += num[i]
            length += 1
        new += 'e+{}'.format(length)
        return new
    
    def pow(self,base,index):
        return 1 if not index else base * self.pow(base,index-1)
        
    def fac(self,facn):
        return 1 if facn==1 else facn * self.fac(facn-1)
    
    def sqrt(self,num):
        accuracy = 0.000000000000001
        low,high = 0,num
        mid = (low+high)/2
        while (high-low)>accuracy:
            if mid**2 > num:
                high = mid
            elif mid**2 < num:
                low = mid
            else:
                return mid
            mid = (low+high)/2
        return mid
    
    def c(self,m,n):
        return self.fac(n) / (self.fac(m) * self.fac(n-m))
    
    def a(self,m,n):
        return self.fac(n) / self.fac(n-m)
    
a = Digit()
print(a.c(5,7))
print(a.a(5,7))



class Data():
    def __init__(self):
        pass
    
    def swap(self,var1,var2):
        return var2,var1
    
    def maxlst(self,lst):
        max = lst[0]
        for i in lst[1:]:
            if i > max:
                max=i
        return max
    
    def minlst(self,lst):
        min = lst[0]
        for i in lst[1:]:
            if i < min:
                min=i
        return min
            
    def randms(self,seed):
        seed **= 2
        return int(str(seed)[1:5])
    
    


class String():
    def __init__(self,__operationstr):
        self.__operationstr = __operationstr
        
    def getstr(self):
        return self.__operationstr
        
    def setstr(self,nowstr):
        self.__operationstr = nowstr
        
    def clearstr(self):
        self.__operationstr = ''
        
    def split(self,substr=' ',amount=-1,rm_empty_char=True,splitmode=None,sortword=False,setword=False):
        if self.__operationstr[-1] != substr:
            self.__operationstr += substr
        split_list = []
        last_search_pos = 0
        num = 0 if amount != -1 else -1
        substr_pos = self.__operationstr.find(substr,last_search_pos)
        while substr_pos != -1 and (num < amount if num != -1 else True):
            nowsplitword = self.__operationstr[last_search_pos:substr_pos]
            last_search_pos =  substr_pos+len(substr)
            substr_pos =  self.__operationstr.find(substr,last_search_pos)
            if num != -1:
                num += 1               
            if rm_empty_char and nowsplitword == '':   
                continue
            if splitmode == None: 
                split_list.append(nowsplitword)
            elif splitmode == 't':
                split_list.append(nowsplitword.title())
            elif splitmode == 'l':
                split_list.append(nowsplitword.lower())
            elif splitmode == 'u':
                split_list.append(nowsplitword.upper())
        if num != -1:
            surplus = self.__operationstr[last_search_pos:-1]
            split_list.append(surplus)
        if setword == True:
            split_list = list(set(split_list))
        if sortword == True:
            split_list.sort()
        self.__operationstr = self.__operationstr[:-1] + ''
        return split_list
    
    def replace(self,replaced_string,replace_string,amount=-1,returntotal=False):
        last_search_pos = 0
        pos_replaced_string = self.__operationstr.find(replaced_string,last_search_pos)
        num = 0 if amount != -1 else -1
        total = 0
        while pos_replaced_string != -1 and (num < amount if num != -1 else True):
            self.__operationstr = self.__operationstr[:pos_replaced_string] + replace_string + self.__operationstr[pos_replaced_string+len(replaced_string):]
            
            last_search_pos =  pos_replaced_string+len(replace_string)
            pos_replaced_string =  self.__operationstr.find(replaced_string,last_search_pos)
            if num != -1:
                num += 1    
            if returntotal:
                total += 1
        if returntotal:
            return self.__operationstr,total
        elif not returntotal:
            return self.__operationstr
        
    def find(self,findstr,start=0,end=-1,case_insensitive=False):
        findlst = []
        total = 0
        if case_insensitive:
            self.__operationstr = self.__operationstr.lower()
            findstr = findstr.lower()
        for i in range(start+1,len(self.__operationstr) if end==-1 else end):
            if self.__operationstr[i:i+len(findstr)] == findstr:
                findlst.append(i)
                total += 1
        return findlst,total
    
    def append(self,appendstr):
        self.__operationstr += appendstr
        
    def insert(self,pos,insertstr):
        self.__operationstr = self.__operationstr[:pos] + insertstr + self.__operationstr[pos:]
        
    def add(self,addstr):
        self.__operationstr = addstr + self.__operationstr
        
    def delete(self,pos,len=1):
        self.__operationstr = self.__operationstr[:pos] + self.__operationstr[pos+len:]
            
    def pop(self,len=1):
        if len > 1:
            pop = self.__operationstr[-1-len:-1]
            self.__operationstr = self.__operationstr[:-1-len]   
            return pop
        elif len==1:
            pop = self.__operationstr[-1]
            self.__operationstr = self.__operationstr[:-1]
            return pop
        
    def count(self,countstr):
        return self.find(countstr)[1]
    
    def reverse(self):
        newstr = ''
        i = len(self.__operationstr)-1
        while i != -1:
            newstr += self.__operationstr[i]
            i -= 1
        self.__operationstr = newstr
        
    def chartotal(self,start=0,end=-1):
        sum = 0
        for i in self.__operationstr[start:len(self.__operationstr) if end==-1 else end]:
            sum += ord(i)
        return sum
    
    def dateformat(self,formatchar='-',timechar=':'):
        if self.__operationstr == "".join(self.split()):
            return False
        else:
            datelst = self.split()
        if len(datelst) == 3:
            return "{}".format(formatchar).join(datelst)
        elif len(datelst) > 3 and  timechar != None:
            return "{}".format(formatchar).join(datelst[:3]) + " " + "{}".format(timechar).join(datelst[3:])
        
    def tobool(self):
        return True if self.__operationstr != '' else False
    
    def toascii(self):
        digitstr = ''
        for i in range(0,len(self.__operationstr)):
            digitstr += str(ord(self.__operationstr[i]))
        return int(digitstr)
    
    def cut(self,num):
        partlen = len(self.__operationstr)//num
        returnlst = []
        head = 0
        for i in range(0,len(self.__operationstr)):
            if self.__operationstr[head:partlen+head] == '':
                return returnlst
            returnlst.append(self.__operationstr[head:partlen+head])
            head += partlen


class Help():
    def __init__(self,helpobject):
        self.ho = helpobject
        self.helpjudge()
        
    def helpjudge(self):
        import pyfol as pfl
        self.ho = pfl.String(self.ho).split("/")
        if self.ho[0] == 'String':
            if self.ho[1] == 'setstr':
                self.Hsetstr()
            elif self.ho[1] == 'clearstr':
                self.Hclearstr()
            elif self.ho[1] == 'split':
                self.Hsplit()
            elif self.ho[1] == 'replace':
                self.Hreplace()
            elif self.ho[1] == 'find':
                self.Hfind()
            elif self.ho[1] == 'append':
                self.Happend()
            elif self.ho[1] == 'insert':
                self.Hinsert()
            elif self.ho[1] == 'add':
                self.Hadd()
            elif self.ho[1] == 'delete':
                self.Hdelete()
            elif self.ho[1] == 'pop':
                self.Hpop()
            elif self.ho[1] == 'count':
                self.Hcount()
            elif self.ho[1] == 'reverse':
                self.Hreverse()
            elif self.ho[1] == 'chartotal':
                self.Hchartotal()
                
    def Hsetstr(self):
        return "Setstr is a function that resets a string to a new string"
        
    def Hclearstr(self):
        return "Clearstr is a function that resets a string to an empty string"
        
    def Hsplit(self):
        return "Split is a function that splits a string"
    
    def Hreplace(self):
        return "Replace is a function that replaces a substring of a string"
        
    def Hfind(self):
        return "Find is a function that finds a substring in a string"
    
    def Happend(self):
        return "Append is a function that adds a new string (or character) to the end of a string"
        
    def Hinsert(self):
        return "Insert is a function that adds a new string (or character) at a specific position of a string"
        
    def Hadd(self):
        return "Add is a function that adds a new string (or character) at the beginning of a string"
        
    def Hdelete(self):
        return "Delete is a function that deletes a certain length of string (or character) at a specific position of a string"
        
    def Hpop(self):
        return "Pop is a function that deletes and pops a certain length of string (or character) at the end of a string"
        
    def Hcount(self):
        return "Count is a function that counts the number of occurrences of substrings in a string"
        
    def Hreverse(self):
        return "Reverse is a function that reverses a string as a whole"
    
    def Hchartotal(self):
        return "Chartotal is a function that counts the ASCII encoding of each character in the function of overall inversion of a string"
        



import codecs
import os
import sys

try:
    from setuptools import setup
except:
    from distutils.core import setup

def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()



NAME = "PyFOL"

PACKAGES = ['pyfol']

DESCRIPTION = "A library that optimizes Python functions"

LONG_DESCRIPTION = read("README.rst")

KEYWORDS = "PyFOL python3 python py FOL"

AUTHOR = "AndyPomeloMars"

AUTHOR_EMAIL = "andypomelomars@163.com"

URL = "https://pypi.python.org/pypi/PyFOL"

VERSION = "1.1.7.3"

LICENSE = "GPL"

setup(
    name = NAME,
    version = VERSION,
    description = DESCRIPTION,
    long_description = LONG_DESCRIPTION,
    classifiers = [
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ],
    keywords = KEYWORDS,
    author = AUTHOR,
    author_email = AUTHOR_EMAIL,
    url = URL,
    license = LICENSE,
    packages = PACKAGES,
    python_requires='>=3.7',
    install_requires = ['prettytable'],
    include_package_data=True,
    zip_safe=True,
)
